@@ -1,8 +1,8 @@
 /**************************************************************************
 **
-** Copyright (c) 2014 BogDan Vatra <bog_dan_ro@yahoo.com>
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 BogDan Vatra <bog_dan_ro@yahoo.com>
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -10,8 +10,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -39,12 +39,10 @@ QT_BEGIN_NAMESPACE
 namespace Ui { class AndroidDeployQtWidget; }
 QT_END_NAMESPACE
 
-namespace QmakeProjectManager { class QmakeBuildConfiguration; }
-
 namespace Android {
 namespace Internal {
 class AndroidDeployQtStep;
-class AndroidExtraLibraryListModel;
+
 class AndroidDeployQtWidget : public ProjectExplorer::BuildStepConfigWidget
 {
     Q_OBJECT
@@ -54,39 +52,16 @@ public:
     ~AndroidDeployQtWidget();
 
 private slots:
-    void setTargetSdk(const QString &sdk);
-    void setMinistro();
-    void setDeployLocalQtLibs();
-    void setBundleQtLibs();
     void installMinistro();
     void cleanLibsOnDevice();
     void resetDefaultDevices();
-    void createKeyStore();
-    void certificatesAliasComboBoxCurrentIndexChanged(const QString &alias);
-    void certificatesAliasComboBoxActivated(const QString &alias);
-    void activeBuildConfigurationChanged();
-    void updateSigningWarning();
-    void openPackageLocationCheckBoxToggled(bool checked);
-    void verboseOutputCheckBoxToggled(bool checked);
-    void updateKeyStorePath(const QString &path);
-    void signPackageCheckBoxToggled(bool checked);
-    void updateInputFileUi();
-    void inputFileComboBoxIndexChanged();
-    void createManifestButton();
-    void addAndroidExtraLib();
-    void removeAndroidExtraLib();
-    void checkEnableRemoveButton();
 
 private:
     virtual QString summaryText() const;
     virtual QString displayName() const;
-    void setCertificates();
 
     Ui::AndroidDeployQtWidget *m_ui;
     AndroidDeployQtStep *m_step;
-    AndroidExtraLibraryListModel *m_extraLibraryListModel;
-    QmakeProjectManager::QmakeBuildConfiguration *m_currentBuildConfiguration;
-    bool m_ignoreChange;
 };
 
 }
