@@ -1,9 +1,9 @@
 /**************************************************************************
 **
-** Copyright (C) 2014 Openismus GmbH.
+** Copyright (C) 2015 Openismus GmbH.
 ** Authors: Peter Penz (ppenz@openismus.com)
 **          Patricia Santana Cruz (patriciasantanacruz@gmail.com)
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -11,20 +11,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -49,9 +50,11 @@
 #include <projectexplorer/target.h>
 #include <projectexplorer/headerpath.h>
 #include <extensionsystem/pluginmanager.h>
-#include <cpptools/cppmodelmanagerinterface.h>
+#include <cpptools/cppmodelmanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/icontext.h>
+#include <qtsupport/baseqtversion.h>
+#include <qtsupport/qtkitinformation.h>
 #include <utils/qtcassert.h>
 #include <utils/filesystemwatcher.h>
 
@@ -142,8 +145,8 @@ bool AutotoolsProject::fromMap(const QVariantMap &map)
     if (!Project::fromMap(map))
         return false;
 
-    connect(m_fileWatcher, SIGNAL(fileChanged(QString)),
-            this, SLOT(onFileChanged(QString)));
+    connect(m_fileWatcher, &Utils::FileSystemWatcher::fileChanged,
+            this, &AutotoolsProject::onFileChanged);
 
     // Load the project tree structure.
     loadProjectTree();
@@ -161,8 +164,8 @@ void AutotoolsProject::loadProjectTree()
         // The thread is still busy parsing a previus configuration.
         // Wait until the thread has been finished and delete it.
         // TODO: Discuss whether blocking is acceptable.
-        disconnect(m_makefileParserThread, SIGNAL(finished()),
-                   this, SLOT(makefileParsingFinished()));
+        disconnect(m_makefileParserThread, &QThread::finished,
+                   this, &AutotoolsProject::makefileParsingFinished);
         m_makefileParserThread->wait();
         delete m_makefileParserThread;
         m_makefileParserThread = 0;
@@ -171,11 +174,11 @@ void AutotoolsProject::loadProjectTree()
     // Parse the makefile asynchronously in a thread
     m_makefileParserThread = new MakefileParserThread(m_fileName);
 
-    connect(m_makefileParserThread, SIGNAL(started()),
-            this, SLOT(makefileParsingStarted()));
+    connect(m_makefileParserThread, &MakefileParserThread::started,
+            this, &AutotoolsProject::makefileParsingStarted);
 
-    connect(m_makefileParserThread, SIGNAL(finished()),
-            this, SLOT(makefileParsingFinished()));
+    connect(m_makefileParserThread, &MakefileParserThread::finished,
+            this, &AutotoolsProject::makefileParsingFinished);
     m_makefileParserThread->start();
 }
 
@@ -271,7 +274,7 @@ void AutotoolsProject::buildFileNodeTree(const QDir &directory,
     // nodes later.
     QHash<QString, Node *> nodeHash;
     foreach (Node * node, nodes(m_rootNode))
-        nodeHash.insert(node->path(), node);
+        nodeHash.insert(node->path().toString(), node);
 
     // Add the sources to the filenode project tree. Sources
     // inside the same directory are grouped into a folder-node.
@@ -281,7 +284,7 @@ void AutotoolsProject::buildFileNodeTree(const QDir &directory,
     FolderNode *parentFolder = 0;
     FolderNode *oldParentFolder = 0;
 
-    foreach (const QString& file, files) {
+    foreach (const QString &file, files) {
         if (file.endsWith(QLatin1String(".moc")))
             continue;
 
@@ -303,8 +306,8 @@ void AutotoolsProject::buildFileNodeTree(const QDir &directory,
                 parentFolder = m_rootNode;
             }
         }
-        QTC_ASSERT(parentFolder != 0, return);
-        if ((oldParentFolder != parentFolder) && !fileNodes.isEmpty()) {
+        QTC_ASSERT(parentFolder, return);
+        if (oldParentFolder && (oldParentFolder != parentFolder) && !fileNodes.isEmpty()) {
             // AutotoolsProjectNode::addFileNodes() is a very expensive operation. It is
             // important to collect as much file nodes of the same parent folder as
             // possible before invoking it.
@@ -316,15 +319,16 @@ void AutotoolsProject::buildFileNodeTree(const QDir &directory,
         const QString filePath = directory.absoluteFilePath(file);
         if (nodeHash.contains(filePath)) {
             nodeHash.remove(filePath);
+        } else if (file == QLatin1String("Makefile.am") || file == QLatin1String("configure.ac")) {
+            fileNodes.append(new FileNode(Utils::FileName::fromString(filePath),
+                                          ProjectFileType, false));
         } else {
-            if (file == QLatin1String("Makefile.am") || file == QLatin1String("configure.ac"))
-                fileNodes.append(new FileNode(filePath, ProjectFileType, false));
-            else
-                fileNodes.append(new FileNode(filePath, ResourceType, false));
+            fileNodes.append(new FileNode(Utils::FileName::fromString(filePath),
+                                          ResourceType, false));
         }
     }
 
-    if (!fileNodes.isEmpty())
+    if (parentFolder && !fileNodes.isEmpty())
         parentFolder->addFileNodes(fileNodes);
 
     // Remove unused file nodes and empty folder nodes
@@ -350,9 +354,9 @@ void AutotoolsProject::buildFileNodeTree(const QDir &directory,
 
 FolderNode *AutotoolsProject::insertFolderNode(const QDir &nodeDir, QHash<QString, Node *> &nodes)
 {
-    const QString nodePath = nodeDir.absolutePath();
-    QFileInfo rootInfo(m_rootNode->path());
-    const QString rootPath = rootInfo.absolutePath();
+    const Utils::FileName nodePath = Utils::FileName::fromString(nodeDir.absolutePath());
+    QFileInfo rootInfo = m_rootNode->path().toFileInfo();
+    const Utils::FileName rootPath = Utils::FileName::fromString(rootInfo.absolutePath());
 
     // Do not create a folder for the root node
     if (rootPath == nodePath)
@@ -378,7 +382,7 @@ FolderNode *AutotoolsProject::insertFolderNode(const QDir &nodeDir, QHash<QStrin
     }
 
     parentFolder->addFolderNodes(QList<FolderNode *>() << folder);
-    nodes.insert(nodePath, folder);
+    nodes.insert(nodePath.toString(), folder);
 
     return folder;
 }
@@ -398,43 +402,61 @@ QList<Node *> AutotoolsProject::nodes(FolderNode *parent) const
     return list;
 }
 
+static QStringList filterIncludes(const QString &absSrc, const QString &absBuild,
+                                  const QStringList &in)
+{
+    QStringList result;
+    foreach (const QString i, in) {
+        QString out = i;
+        out.replace(QLatin1String("$(top_srcdir)"), absSrc);
+        out.replace(QLatin1String("$(abs_top_srcdir)"), absSrc);
+
+        out.replace(QLatin1String("$(top_builddir)"), absBuild);
+        out.replace(QLatin1String("$(abs_top_builddir)"), absBuild);
+
+        result << out;
+    }
+
+    return result;
+}
+
 void AutotoolsProject::updateCppCodeModel()
 {
-    CppTools::CppModelManagerInterface *modelManager =
-        CppTools::CppModelManagerInterface::instance();
-    if (!modelManager)
-        return;
+    CppTools::CppModelManager *modelManager = CppTools::CppModelManager::instance();
 
+    m_codeModelFuture.cancel();
+    CppTools::ProjectInfo pInfo(this);
+    CppTools::ProjectPartBuilder ppBuilder(pInfo);
+
+    CppTools::ProjectPart::QtVersion activeQtVersion = CppTools::ProjectPart::NoQt;
+    if (QtSupport::BaseQtVersion *qtVersion =
+            QtSupport::QtKitInformation::qtVersion(activeTarget()->kit())) {
+        if (qtVersion->qtVersion() < QtSupport::QtVersionNumber(5,0,0))
+            activeQtVersion = CppTools::ProjectPart::Qt4;
+        else
+            activeQtVersion = CppTools::ProjectPart::Qt5;
+    }
+
+    ppBuilder.setQtVersion(activeQtVersion);
     const QStringList cflags = m_makefileParserThread->cflags();
     QStringList cxxflags = m_makefileParserThread->cxxflags();
     if (cxxflags.isEmpty())
         cxxflags = cflags;
+    ppBuilder.setCFlags(cflags);
+    ppBuilder.setCxxFlags(cxxflags);
 
-    CppTools::CppModelManagerInterface::ProjectInfo pinfo = modelManager->projectInfo(this);
-    pinfo.clearProjectParts();
-    CppTools::ProjectPart::Ptr part(new CppTools::ProjectPart);
-    part->project = this;
-    part->displayName = displayName();
-    part->projectFile = projectFilePath().toString();
+    const QString absSrc = projectDirectory().toString();
+    const Target *target = activeTarget();
+    const QString absBuild = (target && target->activeBuildConfiguration())
+            ? target->activeBuildConfiguration()->buildDirectory().toString() : QString();
 
-    if (activeTarget()) {
-        ProjectExplorer::Kit *k = activeTarget()->kit();
-        ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(k);
-        part->evaluateToolchain(tc, cxxflags, cflags,
-                                SysRootKitInformation::sysRoot(k));
-    }
+    ppBuilder.setIncludePaths(filterIncludes(absSrc, absBuild, m_makefileParserThread->includePaths()));
+    ppBuilder.setDefines(m_makefileParserThread->defines());
 
-    foreach (const QString &file, m_files)
-        part->files << CppTools::ProjectFile(file, CppTools::ProjectFile::CXXSource);
+    const QList<Core::Id> languages = ppBuilder.createProjectPartsForFiles(m_files);
+    foreach (Core::Id language, languages)
+        setProjectLanguage(language, true);
 
-    foreach (const QString &inc, m_makefileParserThread->includePaths()) {
-        part->headerPaths += CppTools::ProjectPart::HeaderPath(
-                    inc, CppTools::ProjectPart::HeaderPath::IncludePath);
-    }
-    part->projectDefines += m_makefileParserThread->defines();
-    pinfo.appendProjectPart(part);
-
-    modelManager->updateProjectInfo(pinfo);
-
-    setProjectLanguage(ProjectExplorer::Constants::LANG_CXX, !part->files.isEmpty());
+    pInfo.finish();
+    m_codeModelFuture = modelManager->updateProjectInfo(pInfo);
 }
