@@ -239,7 +239,7 @@ public:
             this, SLOT(provideNewRange(quint64)));
     }
 
-    bool setContents(const QByteArray &contents)
+    bool setContents(const QByteArray &contents) override
     {
         if (!contents.isEmpty())
             return false;
@@ -247,13 +247,13 @@ public:
         return true;
     }
 
-    ReloadBehavior reloadBehavior(ChangeTrigger state, ChangeType type) const
+    ReloadBehavior reloadBehavior(ChangeTrigger state, ChangeType type) const override
     {
         Q_UNUSED(state)
         return type == TypeRemoved ? BehaviorSilent : BehaviorAsk;
     }
 
-    bool save(QString *errorString, const QString &fn, bool autoSave)
+    bool save(QString *errorString, const QString &fn, bool autoSave) override
     {
         QTC_ASSERT(!autoSave, return true); // bineditor does not support autosave - it would be a bit expensive
         const FileName fileNameToUse = fn.isEmpty() ? filePath() : FileName::fromString(fn);
@@ -265,7 +265,15 @@ public:
         }
     }
 
-    bool open(QString *errorString, const QString &fileName, quint64 offset = 0) {
+    OpenResult open(QString *errorString, const QString &fileName,
+                    const QString &realFileName) override
+    {
+        QTC_CHECK(fileName == realFileName); // The bineditor can do no autosaving
+        return openImpl(errorString, fileName);
+    }
+
+    OpenResult openImpl(QString *errorString, const QString &fileName, quint64 offset = 0)
+    {
         QFile file(fileName);
         if (file.open(QIODevice::ReadOnly)) {
             file.close();
@@ -276,7 +284,7 @@ public:
                     *errorString = msg;
                 else
                     QMessageBox::critical(ICore::mainWindow(), tr("File Error"), msg);
-                return false;
+                return OpenResult::CannotHandle;
             }
             if (size > INT_MAX) {
                 QString msg = tr("The file is too big for the Binary Editor (max. 2GB).");
@@ -284,13 +292,13 @@ public:
                     *errorString = msg;
                 else
                     QMessageBox::critical(ICore::mainWindow(), tr("File Error"), msg);
-                return false;
+                return OpenResult::CannotHandle;
             }
             if (offset >= size)
-                return false;
+                return OpenResult::CannotHandle;
             setFilePath(FileName::fromString(fileName));
             m_widget->setSizes(offset, file.size());
-            return true;
+            return OpenResult::Success;
         }
         QString errStr = tr("Cannot open %1: %2").arg(
                 QDir::toNativeSeparators(fileName), file.errorString());
@@ -298,7 +306,7 @@ public:
             *errorString = errStr;
         else
             QMessageBox::critical(ICore::mainWindow(), tr("File Error"), errStr);
-        return false;
+        return OpenResult::ReadError;
     }
 
 private slots:
@@ -326,28 +334,31 @@ private slots:
 
     void provideNewRange(quint64 offset)
     {
-        open(0, filePath().toString(), offset);
+        openImpl(0, filePath().toString(), offset);
     }
 
 public:
 
-    QString defaultPath() const { return QString(); }
+    QString defaultPath() const override { return QString(); }
 
-    QString suggestedFileName() const { return QString(); }
+    QString suggestedFileName() const override { return QString(); }
 
-    bool isModified() const { return isTemporary()/*e.g. memory view*/ ? false
-                                                                       : m_widget->isModified(); }
+    bool isModified() const override
+    {
+        return isTemporary()/*e.g. memory view*/ ? false
+                                                 : m_widget->isModified();
+    }
 
-    bool isFileReadOnly() const {
+    bool isFileReadOnly() const override {
         const FileName fn = filePath();
         if (fn.isEmpty())
             return false;
         return !fn.toFileInfo().isWritable();
     }
 
-    bool isSaveAsAllowed() const { return true; }
+    bool isSaveAsAllowed() const override { return true; }
 
-    bool reload(QString *errorString, ReloadFlag flag, ChangeType type) {
+    bool reload(QString *errorString, ReloadFlag flag, ChangeType type) override {
         if (flag == FlagIgnore)
             return true;
         if (type == TypePermissions) {
@@ -356,7 +367,7 @@ public:
             emit aboutToReload();
             int cPos = m_widget->cursorPosition();
             m_widget->clear();
-            const bool success = open(errorString, filePath().toString());
+            const bool success = (openImpl(errorString, filePath().toString()) == OpenResult::Success);
             m_widget->setCursorPosition(cPos);
             emit reloadFinished(success);
             return success;
@@ -404,18 +415,14 @@ public:
         updateCursorPosition(widget->cursorPosition());
     }
 
-    ~BinEditor()
+    ~BinEditor() override
     {
         delete m_widget;
     }
 
-    bool open(QString *errorString, const QString &fileName, const QString &realFileName) {
-        QTC_ASSERT(fileName == realFileName, return false); // The bineditor can do no autosaving
-        return m_file->open(errorString, fileName);
-    }
-    IDocument *document() { return m_file; }
+    IDocument *document() override { return m_file; }
 
-    QWidget *toolBar() { return m_toolBar; }
+    QWidget *toolBar() override { return m_toolBar; }
 
 private slots:
     void updateCursorPosition(int position) {
@@ -464,7 +471,7 @@ IEditor *BinEditorFactory::createEditor()
 
 ///////////////////////////////// BinEditorPlugin //////////////////////////////////
 
-BinEditorPlugin::BinEditorPlugin() : m_factory(0)
+BinEditorPlugin::BinEditorPlugin()
 {
     m_undoAction = m_redoAction = m_copyAction = m_selectAllAction = 0;
 }
