@@ -61,6 +61,9 @@ public:
 
     QString makePatch(bool revert, bool addPrefix = false) const;
 
+    static Core::IDocument *findOrCreateDocument(const QString &vcsId, const QString &displayName);
+    static DiffEditorController *controller(Core::IDocument *document);
+
 public slots:
     void informationForCommitReceived(const QString &output);
 
@@ -73,7 +76,7 @@ protected:
     // inside reload() (for synchronous reload)
     // or later (for asynchronous reload)
     virtual void reload() = 0;
-    void reloadFinished(bool success);
+    virtual void reloadFinished(bool success);
 
     void setDiffFiles(const QList<FileData> &diffFileList,
                       const QString &baseDirectory = QString());
