@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -32,13 +32,15 @@
 #include "completionassistprovider.h"
 #include "quickfixassistprovider.h"
 #include "iassistprocessor.h"
+#include "textdocument.h"
 #include "iassistproposal.h"
+#include "iassistproposalmodel.h"
 #include "iassistproposalwidget.h"
-#include "iassistinterface.h"
-#include "iassistproposalitem.h"
+#include "assistinterface.h"
+#include "assistproposalitem.h"
 #include "runner.h"
 
-#include <texteditor/basetexteditor.h>
+#include <texteditor/texteditor.h>
 #include <texteditor/texteditorsettings.h>
 #include <texteditor/completionsettings.h>
 #include <coreplugin/editormanager/editormanager.h>
@@ -51,37 +53,16 @@
 #include <QScopedPointer>
 #include <QTimer>
 
-using namespace TextEditor;
-using namespace Internal;
-
-namespace {
-
-template <class T>
-void filterEditorSpecificProviders(QList<T *> *providers, Core::Id editorId)
-{
-    typename QList<T *>::iterator it = providers->begin();
-    while (it != providers->end()) {
-        if ((*it)->supportsEditor(editorId))
-            ++it;
-        else
-            it = providers->erase(it);
-    }
-}
-
-} // Anonymous
+using namespace TextEditor::Internal;
 
 namespace TextEditor {
 
 class CodeAssistantPrivate : public QObject
 {
-    Q_OBJECT
-
 public:
     CodeAssistantPrivate(CodeAssistant *assistant);
-    virtual ~CodeAssistantPrivate();
 
-    void configure(BaseTextEditor *textEditor);
-    void reconfigure();
+    void configure(TextEditorWidget *editorWidget);
     bool isConfigured() const;
 
     void invoke(AssistKind kind, IAssistProvider *provider = 0);
@@ -101,30 +82,26 @@ public:
 
     void stopAutomaticProposalTimer();
     void startAutomaticProposalTimer();
+    void automaticProposalTimeout();
+    void clearAbortedPosition();
+    void updateFromCompletionSettings(const TextEditor::CompletionSettings &settings);
 
     virtual bool eventFilter(QObject *o, QEvent *e);
 
-signals:
-    void finished();
-
-private slots:
-    void finalizeRequest();
+private:
     void proposalComputed();
-    void processProposalItem(IAssistProposalItem *proposalItem);
+    void processProposalItem(AssistProposalItem *proposalItem);
     void handlePrefixExpansion(const QString &newPrefix);
     void finalizeProposal();
-    void automaticProposalTimeout();
-    void updateCompletionSettings(const TextEditor::CompletionSettings &settings);
     void explicitlyAborted();
-    void clearAbortedPosition();
 
 private:
-    CodeAssistant *m_q;
-    BaseTextEditor *m_textEditor;
-    CompletionAssistProvider *m_completionProvider;
+    CodeAssistant *q;
+    TextEditorWidget *m_editorWidget;
     QList<QuickFixAssistProvider *> m_quickFixProviders;
     Internal::ProcessorRunner *m_requestRunner;
     IAssistProvider *m_requestProvider;
+    IAssistProcessor *m_asyncProcessor;
     AssistKind m_assistKind;
     IAssistProposalWidget *m_proposalWidget;
     QScopedPointer<IAssistProposal> m_proposal;
@@ -135,70 +112,58 @@ private:
     static const QChar m_null;
 };
 
-} // TextEditor
-
 // --------------------
 // CodeAssistantPrivate
 // --------------------
 const QChar CodeAssistantPrivate::m_null;
 
-static const int AutomaticProposalTimerInterval = 400;
-
 CodeAssistantPrivate::CodeAssistantPrivate(CodeAssistant *assistant)
-    : m_q(assistant)
-    , m_textEditor(0)
-    , m_completionProvider(0)
+    : q(assistant)
+    , m_editorWidget(0)
     , m_requestRunner(0)
     , m_requestProvider(0)
+    , m_asyncProcessor(0)
     , m_assistKind(TextEditor::Completion)
     , m_proposalWidget(0)
     , m_receivedContentWhileWaiting(false)
-    , m_settings(TextEditorSettings::completionSettings())
     , m_abortedBasePosition(-1)
 {
     m_automaticProposalTimer.setSingleShot(true);
-    m_automaticProposalTimer.setInterval(AutomaticProposalTimerInterval);
-    connect(&m_automaticProposalTimer, SIGNAL(timeout()), this, SLOT(automaticProposalTimeout()));
+    connect(&m_automaticProposalTimer, &QTimer::timeout,
+            this, &CodeAssistantPrivate::automaticProposalTimeout);
 
-    connect(TextEditorSettings::instance(),
-            SIGNAL(completionSettingsChanged(TextEditor::CompletionSettings)),
-            this,
-            SLOT(updateCompletionSettings(TextEditor::CompletionSettings)));
-    connect(Core::EditorManager::instance(), SIGNAL(currentEditorChanged(Core::IEditor*)),
-            this, SLOT(clearAbortedPosition()));
+    connect(TextEditorSettings::instance(), &TextEditorSettings::completionSettingsChanged,
+            this, &CodeAssistantPrivate::updateFromCompletionSettings);
+
+    connect(Core::EditorManager::instance(), &Core::EditorManager::currentEditorChanged,
+            this, &CodeAssistantPrivate::clearAbortedPosition);
 }
 
-CodeAssistantPrivate::~CodeAssistantPrivate()
-{
-}
-
-void CodeAssistantPrivate::configure(BaseTextEditor *textEditor)
+void CodeAssistantPrivate::configure(TextEditorWidget *editorWidget)
 {
     // @TODO: There's a list of providers but currently only the first one is used. Perhaps we
     // should implement a truly mechanism to support multiple providers for an editor (either
     // merging or not proposals) or just leave it as not extensible and store directly the one
     // completion and quick-fix provider (getting rid of the list).
 
-    m_textEditor = textEditor;
-    m_completionProvider = textEditor->completionAssistProvider();
-    m_quickFixProviders =
-        ExtensionSystem::PluginManager::getObjects<QuickFixAssistProvider>();
-    filterEditorSpecificProviders(&m_quickFixProviders, m_textEditor->document()->id());
+    m_editorWidget = editorWidget;
+    m_quickFixProviders = ExtensionSystem::PluginManager::getObjects<QuickFixAssistProvider>();
 
-    m_textEditor->editorWidget()->installEventFilter(this);
-    connect(m_textEditor->baseTextDocument(),SIGNAL(mimeTypeChanged()),
-            m_q, SLOT(reconfigure()));
-}
+    Core::Id editorId = m_editorWidget->textDocument()->id();
+    auto it = m_quickFixProviders.begin();
+    while (it != m_quickFixProviders.end()) {
+        if ((*it)->supportsEditor(editorId))
+            ++it;
+        else
+            it = m_quickFixProviders.erase(it);
+    }
 
-void CodeAssistantPrivate::reconfigure()
-{
-    if (isConfigured())
-        m_completionProvider = m_textEditor->completionAssistProvider();
+    m_editorWidget->installEventFilter(this);
 }
 
 bool CodeAssistantPrivate::isConfigured() const
 {
-    return m_textEditor != 0;
+    return m_editorWidget != 0;
 }
 
 void CodeAssistantPrivate::invoke(AssistKind kind, IAssistProvider *provider)
@@ -210,10 +175,9 @@ void CodeAssistantPrivate::invoke(AssistKind kind, IAssistProvider *provider)
 
     if (isDisplayingProposal() && m_assistKind == kind && !m_proposal->isFragile()) {
         m_proposalWidget->setReason(ExplicitlyInvoked);
-        m_proposalWidget->updateProposal(
-                    m_textEditor->textDocument()->textAt(
+        m_proposalWidget->updateProposal(m_editorWidget->textAt(
                         m_proposal->basePosition(),
-                        m_textEditor->position() - m_proposal->basePosition()));
+                        m_editorWidget->position() - m_proposal->basePosition()));
     } else {
         destroyContext();
         requestProposal(ExplicitlyInvoked, kind, provider);
@@ -249,12 +213,12 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
 {
     QTC_ASSERT(!isWaitingForProposal(), return);
 
-    if (m_textEditor->editorWidget()->hasBlockSelection())
+    if (m_editorWidget->hasBlockSelection())
         return; // TODO
 
     if (!provider) {
         if (kind == Completion)
-            provider = m_completionProvider;
+            provider = m_editorWidget->textDocument()->completionAssistProvider();
         else if (!m_quickFixProviders.isEmpty())
             provider = m_quickFixProviders.at(0);
 
@@ -264,37 +228,76 @@ void CodeAssistantPrivate::requestProposal(AssistReason reason,
 
     m_assistKind = kind;
     IAssistProcessor *processor = provider->createProcessor();
-    IAssistInterface *assistInterface =
-        m_textEditor->editorWidget()->createAssistInterface(kind, reason);
-    if (!assistInterface)
+    AssistInterface *assistInterface = m_editorWidget->createAssistInterface(kind, reason);
+    if (!assistInterface) {
+        delete processor;
         return;
+    }
 
-    if (provider->isAsynchronous()) {
+    switch (provider->runType()) {
+    case IAssistProvider::Synchronous: {
+        if (IAssistProposal *newProposal = processor->perform(assistInterface))
+            displayProposal(newProposal, reason);
+        delete processor;
+        break;
+    }
+    case IAssistProvider::AsynchronousWithThread: {
         if (IAssistProposal *newProposal = processor->immediateProposal(assistInterface))
             displayProposal(newProposal, reason);
 
         m_requestProvider = provider;
         m_requestRunner = new ProcessorRunner;
-        connect(m_requestRunner, SIGNAL(finished()), this, SLOT(proposalComputed()));
-        connect(m_requestRunner, SIGNAL(finished()), this, SLOT(finalizeRequest()));
-        connect(m_requestRunner, SIGNAL(finished()), this, SIGNAL(finished()));
+        connect(m_requestRunner, &ProcessorRunner::finished,
+                this, &CodeAssistantPrivate::proposalComputed);
+        connect(m_requestRunner, &ProcessorRunner::finished,
+                m_requestRunner, &QObject::deleteLater);
+        connect(m_requestRunner, &ProcessorRunner::finished,
+                q, &CodeAssistant::finished);
         assistInterface->prepareForAsyncUse();
         m_requestRunner->setReason(reason);
         m_requestRunner->setProcessor(processor);
         m_requestRunner->setAssistInterface(assistInterface);
         m_requestRunner->start();
-        return;
+        break;
     }
+    case IAssistProvider::Asynchronous: {
+        processor->setAsyncCompletionAvailableHandler(
+            [this, processor, reason](IAssistProposal *newProposal){
+                if (m_asyncProcessor != processor) {
+                    delete newProposal->model();
+                    delete newProposal;
+                    return;
+                }
 
-    if (IAssistProposal *newProposal = processor->perform(assistInterface))
-        displayProposal(newProposal, reason);
-    delete processor;
+                invalidateCurrentRequestData();
+                QTC_CHECK(newProposal);
+                displayProposal(newProposal, reason);
+
+                emit q->finished();
+        });
+
+        // If there is a proposal, nothing asynchronous happened...
+        if (IAssistProposal *newProposal = processor->perform(assistInterface)) {
+            displayProposal(newProposal, reason);
+            delete processor;
+        } else if (!processor->performWasApplicable()) {
+            delete processor;
+        }
+
+        // ...otherwise the async request was triggered
+        m_asyncProcessor = processor;
+        break;
+    }
+    } // switch
 }
 
 void CodeAssistantPrivate::cancelCurrentRequest()
 {
-    m_requestRunner->setDiscardProposal(true);
-    disconnect(m_requestRunner, SIGNAL(finished()), this, SLOT(proposalComputed()));
+    if (m_requestRunner) {
+        m_requestRunner->setDiscardProposal(true);
+        disconnect(m_requestRunner, &ProcessorRunner::finished,
+                   this, &CodeAssistantPrivate::proposalComputed);
+    }
     invalidateCurrentRequestData();
 }
 
@@ -302,7 +305,7 @@ void CodeAssistantPrivate::proposalComputed()
 {
     // Since the request runner is a different thread, there's still a gap in which the queued
     // signal could be processed after an invalidation of the current request.
-    if (m_requestRunner != sender())
+    if (!m_requestRunner || m_requestRunner != sender())
         return;
 
     IAssistProposal *newProposal = m_requestRunner->proposal();
@@ -325,7 +328,7 @@ void CodeAssistantPrivate::displayProposal(IAssistProposal *newProposal, AssistR
     }
 
     int basePosition = proposalCandidate->basePosition();
-    if (m_textEditor->position() < basePosition)
+    if (m_editorWidget->position() < basePosition)
         return;
 
     if (m_abortedBasePosition == basePosition && reason != ExplicitlyInvoked)
@@ -335,36 +338,37 @@ void CodeAssistantPrivate::displayProposal(IAssistProposal *newProposal, AssistR
     m_proposal.reset(proposalCandidate.take());
 
     if (m_proposal->isCorrective())
-        m_proposal->makeCorrection(m_textEditor);
+        m_proposal->makeCorrection(m_editorWidget);
 
     basePosition = m_proposal->basePosition();
     m_proposalWidget = m_proposal->createWidget();
-    connect(m_proposalWidget, SIGNAL(destroyed()), this, SLOT(finalizeProposal()));
-    connect(m_proposalWidget, SIGNAL(prefixExpanded(QString)),
-            this, SLOT(handlePrefixExpansion(QString)));
-    connect(m_proposalWidget, SIGNAL(proposalItemActivated(IAssistProposalItem*)),
-            this, SLOT(processProposalItem(IAssistProposalItem*)));
-    connect(m_proposalWidget, SIGNAL(explicitlyAborted()),
-            this, SLOT(explicitlyAborted()));
-    m_proposalWidget->setAssistant(m_q);
+    connect(m_proposalWidget, &QObject::destroyed,
+            this, &CodeAssistantPrivate::finalizeProposal);
+    connect(m_proposalWidget, &IAssistProposalWidget::prefixExpanded,
+            this, &CodeAssistantPrivate::handlePrefixExpansion);
+    connect(m_proposalWidget, &IAssistProposalWidget::proposalItemActivated,
+            this, &CodeAssistantPrivate::processProposalItem);
+    connect(m_proposalWidget, &IAssistProposalWidget::explicitlyAborted,
+            this, &CodeAssistantPrivate::explicitlyAborted);
+    m_proposalWidget->setAssistant(q);
     m_proposalWidget->setReason(reason);
     m_proposalWidget->setKind(m_assistKind);
-    m_proposalWidget->setUnderlyingWidget(m_textEditor->widget());
+    m_proposalWidget->setUnderlyingWidget(m_editorWidget);
     m_proposalWidget->setModel(m_proposal->model());
-    m_proposalWidget->setDisplayRect(m_textEditor->cursorRect(basePosition));
+    m_proposalWidget->setDisplayRect(m_editorWidget->cursorRect(basePosition));
     if (m_receivedContentWhileWaiting)
         m_proposalWidget->setIsSynchronized(false);
     else
         m_proposalWidget->setIsSynchronized(true);
-    m_proposalWidget->showProposal(m_textEditor->textDocument()->textAt(
+    m_proposalWidget->showProposal(m_editorWidget->textAt(
                                        basePosition,
-                                       m_textEditor->position() - basePosition));
+                                       m_editorWidget->position() - basePosition));
 }
 
-void CodeAssistantPrivate::processProposalItem(IAssistProposalItem *proposalItem)
+void CodeAssistantPrivate::processProposalItem(AssistProposalItem *proposalItem)
 {
     QTC_ASSERT(m_proposal, return);
-    proposalItem->apply(m_textEditor, m_proposal->basePosition());
+    proposalItem->apply(m_editorWidget, m_proposal->basePosition());
     destroyContext();
     process();
 }
@@ -372,18 +376,12 @@ void CodeAssistantPrivate::processProposalItem(IAssistProposalItem *proposalItem
 void CodeAssistantPrivate::handlePrefixExpansion(const QString &newPrefix)
 {
     QTC_ASSERT(m_proposal, return);
-    const int currentPosition = m_textEditor->position();
-    m_textEditor->setCursorPosition(m_proposal->basePosition());
-    m_textEditor->replace(currentPosition - m_proposal->basePosition(), newPrefix);
+    const int currentPosition = m_editorWidget->position();
+    m_editorWidget->setCursorPosition(m_proposal->basePosition());
+    m_editorWidget->replace(currentPosition - m_proposal->basePosition(), newPrefix);
     notifyChange();
 }
 
-void CodeAssistantPrivate::finalizeRequest()
-{
-    if (ProcessorRunner *runner = qobject_cast<ProcessorRunner *>(sender()))
-        delete runner;
-}
-
 void CodeAssistantPrivate::finalizeProposal()
 {
     stopAutomaticProposalTimer();
@@ -400,25 +398,26 @@ bool CodeAssistantPrivate::isDisplayingProposal() const
 
 bool CodeAssistantPrivate::isWaitingForProposal() const
 {
-    return m_requestRunner != 0;
+    return m_requestRunner != 0 || m_asyncProcessor != 0;
 }
 
 void CodeAssistantPrivate::invalidateCurrentRequestData()
 {
+    m_asyncProcessor = 0;
     m_requestRunner = 0;
     m_requestProvider = 0;
 }
 
 CompletionAssistProvider *CodeAssistantPrivate::identifyActivationSequence()
 {
-    if (!m_completionProvider)
+    CompletionAssistProvider *completionProvider = m_editorWidget->textDocument()->completionAssistProvider();
+    if (!completionProvider)
         return 0;
 
-    const int length = m_completionProvider->activationCharSequenceLength();
+    const int length = completionProvider->activationCharSequenceLength();
     if (length == 0)
         return 0;
-    QString sequence = m_textEditor->textDocument()->textAt(m_textEditor->position() - length,
-                                                            length);
+    QString sequence = m_editorWidget->textAt(m_editorWidget->position() - length, length);
     // In pretty much all cases the sequence will have the appropriate length. Only in the
     // case of typing the very first characters in the document for providers that request a
     // length greater than 1 (currently only C++, which specifies 3), the sequence needs to
@@ -426,7 +425,7 @@ CompletionAssistProvider *CodeAssistantPrivate::identifyActivationSequence()
     const int lengthDiff = length - sequence.length();
     for (int j = 0; j < lengthDiff; ++j)
         sequence.prepend(m_null);
-    return m_completionProvider->isActivationCharSequence(sequence) ? m_completionProvider : 0;
+    return completionProvider->isActivationCharSequence(sequence) ? completionProvider : 0;
 }
 
 void CodeAssistantPrivate::notifyChange()
@@ -435,12 +434,12 @@ void CodeAssistantPrivate::notifyChange()
 
     if (isDisplayingProposal()) {
         QTC_ASSERT(m_proposal, return);
-        if (m_textEditor->position() < m_proposal->basePosition()) {
+        if (m_editorWidget->position() < m_proposal->basePosition()) {
             destroyContext();
         } else {
             m_proposalWidget->updateProposal(
-                m_textEditor->textDocument()->textAt(m_proposal->basePosition(),
-                                     m_textEditor->position() - m_proposal->basePosition()));
+                m_editorWidget->textAt(m_proposal->basePosition(),
+                                     m_editorWidget->position() - m_proposal->basePosition()));
             if (m_proposal->isFragile())
                 startAutomaticProposalTimer();
         }
@@ -449,7 +448,7 @@ void CodeAssistantPrivate::notifyChange()
 
 bool CodeAssistantPrivate::hasContext() const
 {
-    return m_requestRunner || m_proposalWidget;
+    return m_requestRunner || m_asyncProcessor || m_proposalWidget;
 }
 
 void CodeAssistantPrivate::destroyContext()
@@ -460,7 +459,8 @@ void CodeAssistantPrivate::destroyContext()
         cancelCurrentRequest();
     } else if (isDisplayingProposal()) {
         m_proposalWidget->closeProposal();
-        disconnect(m_proposalWidget, SIGNAL(destroyed()), this, SLOT(finalizeProposal()));
+        disconnect(m_proposalWidget, &QObject::destroyed,
+                   this, &CodeAssistantPrivate::finalizeProposal);
         finalizeProposal();
     }
 }
@@ -485,9 +485,11 @@ void CodeAssistantPrivate::stopAutomaticProposalTimer()
         m_automaticProposalTimer.stop();
 }
 
-void CodeAssistantPrivate::updateCompletionSettings(const TextEditor::CompletionSettings &settings)
+void CodeAssistantPrivate::updateFromCompletionSettings(
+        const TextEditor::CompletionSettings &settings)
 {
     m_settings = settings;
+    m_automaticProposalTimer.setInterval(m_settings.m_automaticProposalTimeoutInMs);
 }
 
 void CodeAssistantPrivate::explicitlyAborted()
@@ -537,17 +539,22 @@ bool CodeAssistantPrivate::eventFilter(QObject *o, QEvent *e)
 // -------------
 CodeAssistant::CodeAssistant() : d(new CodeAssistantPrivate(this))
 {
-    connect(d, SIGNAL(finished()), SIGNAL(finished()));
 }
 
 CodeAssistant::~CodeAssistant()
 {
+    destroyContext();
     delete d;
 }
 
-void CodeAssistant::configure(BaseTextEditor *textEditor)
+void CodeAssistant::configure(TextEditorWidget *editorWidget)
 {
-    d->configure(textEditor);
+    d->configure(editorWidget);
+}
+
+void CodeAssistant::updateFromCompletionSettings(const CompletionSettings &settings)
+{
+    d->updateFromCompletionSettings(settings);
 }
 
 void CodeAssistant::process()
@@ -575,9 +582,4 @@ void CodeAssistant::invoke(AssistKind kind, IAssistProvider *provider)
     d->invoke(kind, provider);
 }
 
-void CodeAssistant::reconfigure()
-{
-    d->reconfigure();
-}
-
-#include "codeassistant.moc"
+} // namespace TextEditor
